"use strict";
const Energistics = require("../Energistics");
var socket = require('websocket').w3cwebsocket;
class RecordingConsumer {
    constructor(_config) {
        this._config = _config;
        this._connected = false;
        this._running = false;
        this._channels = [];
        this._channelBuffers = [];
        _config || (_config = {});
        _config.retryInterval || (_config.retryInterval = 30000);
        _config.encoding || (_config.encoding = "binary");
        _config.url || (_config.url = "ws://localhost:8080");
        _config.uri || (_config.uri = "/");
        _config.channelPattern || (_config.channelPattern = "^.*$");
        this._etp = new Energistics.ETPClient(_config);
        this._etp.on('log', console.log);
        this._etp.on('connect', this.onSocketConnect.bind(this));
        this._etp.on('disconnect', this.onSocketDisconnect.bind(this));
        this._etp.on('close', this.onCloseSession.bind(this));
        this._etp.on('open', this.onOpenSession.bind(this));
        this._etp.registerHandler(1, new Energistics.ChannelStreamingConsumer(this._etp));
        this._etp.handlers[1].on('metadata', this.onChannelMetadata.bind(this));
        this._etp.handlers[1].on('data', this.onChannelData.bind(this));
    }
    connect() {
        if (this._running) {
            this._etp.connect(this._config, socket);
        }
    }
    onChannelData(msg) {
        var message = msg.body;
        var data = message.data;
        if (this._config.storeAsChannelSet) {
            this._config.store.putIndexAlignedValues(this._channels, this._config.contextUri, data);
        }
        else {
            var indexes = message.data[0].indexes;
            for (var i = 0; i < data.length; i++) {
                var channelId = data[i].channelId;
                var thisChannel = this._channels[channelId];
                if (thisChannel !== undefined) {
                    if (data[i].indexes.length > 0) {
                        indexes = data[i].indexes;
                    }
                    else {
                        data[i].indexes = indexes;
                    }
                    var buffer = this._channelBuffers[channelId];
                    if (buffer.length == 60) {
                        this._channelBuffers[channelId] = [];
                        this._config.store.putChannelValues(thisChannel, buffer);
                    }
                    this._channelBuffers[channelId].push(data[i]);
                }
                else {
                    console.log("Received data point for unregistered channel: " + channelId);
                }
            }
        }
    }
    get connectedToSimpleStreamer() {
        return (this._etp.serverProtocols[1].protocolCapabilities["simpleStreamer"] != undefined);
    }
    onChannelMetadata(msg) {
        var message = msg.body;
        for (var i = 0; i < message.channels.length; i++) {
            var thisChannel = message.channels[i];
            this._channels[thisChannel.channelId] = thisChannel;
            this._channelBuffers[thisChannel.channelId] = [];
            if (this._config.contextUri) {
                thisChannel.channelUri = this._config.contextUri + "/" + thisChannel.channelUri;
                thisChannel.contextUri = this._config.contextUri;
            }
            this._config.store.addChannel(thisChannel, function (err, data) {
                if (!this.consumer.connectedToSimpleStreamer) {
                    if (this.thisChannel.channelName.match(this.consumer._config.channelPattern)) {
                        var csi = new Energistics.Datatypes.ChannelData.ChannelStreamingInfo();
                        csi.channelId = this.thisChannel.channelId;
                        csi.receiveChangeNotification = true;
                        (this.consumer._etp.handlers[1]).channelStreamingStart([csi]);
                    }
                }
            }.bind({ consumer: this, thisChannel: thisChannel }));
        }
    }
    onCloseSession() {
        this._channels = [];
        this._channelBuffers = [];
    }
    onOpenSession() {
        this._channels = [];
        this._channelBuffers = [];
        if (!this.connectedToSimpleStreamer) {
            (this._etp.handlers[1]).describe([this._config.uri]);
        }
    }
    onSocketConnect(socket) {
        this._connected = true;
        console.log('Socket Status: ' + socket.readyState + ' (open)');
        console.log("Requesting session with {" + this._config.url + "}");
        var thisVersion = { major: 1, minor: 0, revision: 0, patch: 0 };
        this._etp.requestSession([
            { protocol: 1, protocolVersion: thisVersion, role: "producer", protocolCapabilities: {} }
        ], "ralf-recording-client");
    }
    onSocketDisconnect(socket) {
        this._connected = false;
    }
    retry() {
        if (!this._connected && this._running) {
            console.log("Trying to connect to: " + this._config.url);
            this.connect();
        }
    }
    start() {
        this._running = true;
        this.connect();
        if (this._config.retryInterval > 1000) {
            setInterval(this.retry.bind(this), this._config.retryInterval);
        }
    }
    stop() {
        this._running = false;
        if (this._connected) {
            this._etp.closeSession();
        }
    }
}
module.exports = RecordingConsumer;
//# sourceMappingURL=RecordingConsumer.js.map