"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const uuid = require('node-uuid');
const avro = require("etp-avro");
const etp_1 = require("etp");
const ETPCore_1 = require("../common/ETPCore");
const EnergisticsCommons_1 = require("../common/EnergisticsCommons");
const Core = etp_1.Energistics.Protocol.Core;
const PROTOCOL = etp_1.Energistics.Datatypes.Protocols;
class ETPClient extends ETPCore_1.ETPCore {
    constructor(config) {
        super(config);
        this.serverProtocols = [];
    }
    static atob(str) {
        return new Buffer(str, 'base64').toString('binary');
    }
    connect(config, socketClass) {
        if (!socketClass)
            socketClass = WebSocket;
        if (config.url != null)
            this.host = config.url;
        var encoding = config.encoding ? config.encoding : "binary";
        this.binary = (encoding === "binary");
        var headers = {
            "etp-encoding": encoding,
            "Authorization": ""
        };
        if (config.authentication == "basic") {
            headers.Authorization = 'Basic ' + new Buffer(config.username + ':' + config.password).toString('base64');
        }
        else if (config.authentication == "jwt") {
            headers.Authorization = 'Bearer ' + config.token;
        }
        if (config.noHeaders) {
            this.host = this.host + "?etp-encoding=" + encoding;
            if (headers.Authorization) {
                this.host = this.host + "&Authorization=" + headers.Authorization;
            }
        }
        this.connection = new socketClass(this.host, "energistics-tp", "localhost", headers);
        if (this.binary)
            this.connection.binaryType = "arraybuffer";
        this.connection.onopen = (function (evt) {
            if (this.connection.readyState == 1) {
                this.emit("connect", this.connection);
            }
        }).bind(this);
        this.connection.onclose = this.onSocketClose.bind(this);
        this.connection.onmessage = this.onSocketMessage.bind(this);
        this.connection.onerror = (function (evt) {
            this.emit('log', 'Connection Error');
        }).bind(this);
    }
    handleMessage(messageHeader, messageBody) {
        if (messageHeader.protocol == PROTOCOL.Core) {
            switch (messageHeader.messageType) {
                case Core.MsgOpenSession:
                    this.onOpenSession(messageHeader, messageBody);
                    break;
                case Core.MsgCloseSession:
                    this.onCloseSession(messageHeader, messageBody);
                    break;
                default:
                    super.handleMessage(messageHeader, messageBody);
            }
        }
        else {
            this.handlers[messageHeader.protocol].handleMessage(messageHeader, messageBody);
        }
    }
    onOpenSession(messageHeader, messageBody) {
        this.log("Opened Session {" + messageBody.sessionId + "} with " + this.host);
        this.sessionId = messageBody.sessionId;
        for (var i = 0; i < messageBody.supportedProtocols.length; i++) {
            var prot = messageBody.supportedProtocols[i];
            var protocolId = prot.protocol;
            this.serverProtocols[protocolId] = prot;
            this.log("Supported protocol: " + protocolId + "; " + prot.role + "; caps: " + JSON.stringify(prot.protocolCapabilities));
            if (this.handlers[protocolId]) {
                this.handlers[protocolId].start();
            }
        }
        this.emit('open');
    }
    onSocketClose(evt) {
        this.sessionId = EnergisticsCommons_1.NO_SESSION;
        this.log((new Date()) + ' Peer ' + this.connection.url + ' disconnected.');
        this.log('--Reason Code [' + evt.code + '] Description [' + evt.reason + ']');
        this.emit('disconnect', this.connection);
    }
    onSocketMessage(msg) {
        var header = null;
        var message = null;
        this.stats.messagesReceived++;
        if (typeof (msg.data) == "object") {
            this.stats.bytesReceived += msg.data.byteLength;
            var reader = new avro.BinaryReader(this.schemaCache, new Uint8Array(msg.data));
            header = reader.readDatum("Energistics.Datatypes.MessageHeader");
            message = reader.readDatum(this.schemaCache.find(header.protocol, header.messageType));
        }
        else {
            this.stats.bytesReceived += msg.data.length;
            var data = JSON.parse(msg.data);
            header = data[0];
            message = data[1];
        }
        this.handleMessage(header, message);
    }
    requestSession(requestedProtocols, applicationName, applicationVersion = "") {
        var header = this.createHeader(PROTOCOL.Core, Core.MsgRequestSession, 0);
        var message = {
            requestedProtocols: requestedProtocols,
            applicationName: applicationName,
            applicationVersion: applicationVersion
        };
        this.send(header, message);
    }
    send(header, message) {
        this.stats.messagesSent++;
        var dataToSend = null, encoder;
        if (this.binary) {
            encoder = new avro.BinaryWriter(this.schemaCache);
            encoder.writeDatum("Energistics.Datatypes.MessageHeader", header);
            if (message) {
                var schemaName = this.schemaCache.find(header.protocol, header.messageType).fullName;
                encoder.writeDatum(schemaName, message);
            }
            dataToSend = encoder.getArrayBuffer();
        }
        else {
            dataToSend = JSON.stringify([header, message]);
        }
        this.stats.bytesSent += dataToSend.byteLength;
        this.connection.send(dataToSend);
        return header.messageId;
    }
}
exports.ETPClient = ETPClient;
//# sourceMappingURL=ETPClient.js.map