"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const events = require("events");
const common = require("../common/EnergisticsCommons");
var Jsonix = require("jsonix").Jsonix;
class LogLoader {
    constructor(store, log, parentLoader) {
        this.store = store;
        this.log = log;
        this.parentLoader = parentLoader;
        this.headerOnly = false;
        this.channelCount = 0;
        this.channelsRegistered = 0;
        this.pivot = false;
        this.curveMap = {};
        this.channelMetadata = [];
        this.channelData = {};
        this.frameData = [];
        this.logData = log.logData;
        this.logCurveInfo = log.logCurveInfo;
        log.logData = null;
        log.logCurveInfo = null;
    }
    dataTypeFromCurveInfo(info) {
        return info.typeLogData;
    }
    channelDataMap() {
        var retVal = {};
        for (var i = 0; i < this.channelMetadata.length; i++) {
            retVal[this.channelMetadata[i]._id] = this.channelMetadata[i];
        }
        return retVal;
    }
    logToChannelIndex(log) {
        var indexMetadata = {};
        var indexCurve;
        for (var i = 0; i < this.logCurveInfo.length; i++) {
            if (log.indexCurve == this.logCurveInfo[i].uid || log.indexCurve == this.logCurveInfo[i].mnemonic.value) {
                indexCurve = this.logCurveInfo[i];
                break;
            }
        }
        switch (log.indexType) {
            case "date time":
                indexMetadata.indexType = "Time";
                break;
            case "measured depth":
                indexMetadata.indexType = "Depth";
                break;
            default:
                indexMetadata = "Time";
        }
        indexMetadata.direction = log.direction.substr(0, 1).toUpperCase() + log.direction.substr(1);
        indexMetadata.datum = null;
        indexMetadata.uom = indexCurve.unit;
        indexMetadata.scale = 5;
        return indexMetadata;
    }
    curveInfoToChannelMetadata(log, info, theIndex) {
        return {
            channelUri: "eml:///witsml14/log(" + log.uid + ")/logCurveInfo(" + info.mnemonic.value + ")",
            contextUri: "eml:///witsml14/log(" + log.uid + ")",
            indexes: [theIndex],
            dataType: this.dataTypeFromCurveInfo(info),
            uom: info.unit,
            startIndex: null,
            endIndex: null,
            channelName: info.mnemonic.value,
            description: info.curveDescription,
            status: "Closed",
            source: log.serviceCompany,
            measureClass: "",
        };
    }
    parseLogCurves() {
        for (var dataSetId = 0; dataSetId < this.logData.length; dataSetId++) {
            var dataSet = this.logData[dataSetId];
            var mnemonicList = dataSet.mnemonicList.split(",");
            var unitList = dataSet.unitList.split(",");
            var indexInfo = this.curveMap[mnemonicList[1]].metadata.indexes[0];
            var rows = dataSet.data;
            if (rows && (rows.length > 0)) {
                for (var rowNum = 0; rowNum < rows.length; rowNum++) {
                    var row = rows[rowNum].split(",");
                    var indexValue = row[0];
                    if (indexInfo.indexType == "Time") {
                        indexValue = common.encodeIndexValue(indexInfo, Date.parse(indexValue));
                    }
                    else {
                        indexValue = common.encodeIndexValue(indexInfo, indexValue);
                    }
                    if (this.pivot) {
                        for (var i = 1; i < row.length; i++) {
                            var channelMetadata = this.curveMap[mnemonicList[i]].metadata;
                            var dataArray = this.channelData[channelMetadata.channelName];
                            if (row[i] != "") {
                                var dataItem = {
                                    indexes: [indexValue],
                                    channelId: channelMetadata._id,
                                    value: { item: { double: parseFloat(row[i]) } },
                                    valueAttributes: []
                                };
                                dataArray.push(dataItem);
                            }
                            else {
                            }
                        }
                    }
                    else {
                        var frame = [];
                        for (var i = 1; i < row.length; i++) {
                            var channelMetadata = this.curveMap[mnemonicList[i]].metadata;
                            var info = this.curveMap[mnemonicList[i]].info;
                            if (row[i] != "" && row[i] != info.nullValue) {
                                var dataItem = {
                                    indexes: [],
                                    channelId: channelMetadata._id,
                                    value: { item: { double: parseFloat(row[i]) } },
                                    valueAttributes: []
                                };
                                frame.push(dataItem);
                            }
                            else {
                                this.parentLoader.nullValues++;
                            }
                        }
                        if (frame.length > 0) {
                            frame[0].indexes.push(indexValue);
                            this.frameData.push(frame);
                        }
                    }
                }
            }
        }
    }
    insertLogCurves() {
        var done = 0;
        if (this.pivot) {
            for (var q = 0; q < this.channelMetadata.length; q++) {
                var channelInfo = this.channelMetadata[q];
                var curveName = channelInfo.channelName;
                var data = this.channelData[curveName];
                if (data && (data.length > 0)) {
                    this.store.putChannelValues(channelInfo, data, function () {
                        done++;
                        if (done == this.channelMetadata.length) {
                            this.parentLoader.emit('done');
                        }
                    }.bind(this));
                }
                else {
                    this.parentLoader.emit('done');
                }
            }
        }
        else {
            this.store.putIndexAlignedValuesBatch(this.channelDataMap(), { bulk: true }, this.frameData, function (err, result) {
                this.parentLoader.emit('done');
            }.bind(this));
        }
    }
    loadLog() {
        this.curveMap = {};
        this.channelData = {};
        this.channelMetadata = [];
        if (!this.logCurveInfo) {
            this.parentLoader.emit('done');
            return;
        }
        var theIndex = this.logToChannelIndex(this.log);
        this.channelCount = this.logCurveInfo.length;
        this.channelsRegistered = 0;
        for (var i = 0; i < this.channelCount; i++) {
            var info = this.logCurveInfo[i];
            this.curveMap[info.mnemonic.value] = { info: info };
            if (info.mnemonic.value != this.log.indexCurve) {
                var channelInfo = this.curveInfoToChannelMetadata(this.log, info, theIndex);
                this.channelMetadata.push(channelInfo);
                if (this.store) {
                    this.store.addChannel(channelInfo, function (err, data) {
                        this.channelsRegistered++;
                        if (this.channelsRegistered == (this.channelCount - 1)) {
                            if (this.parentLoader.headerOnly == true) {
                                this.parentLoader.emit("done");
                            }
                            else {
                                this.parseLogCurves();
                                this.insertLogCurves();
                            }
                        }
                    }.bind(this));
                }
                else {
                    channelInfo.channelId = channelInfo._id = i;
                }
                this.curveMap[info.mnemonic.value].metadata = channelInfo;
                this.channelData[info.mnemonic.value] = [];
            }
        }
        if (!this.store) {
            this.parseLogCurves();
        }
    }
}
exports.LogLoader = LogLoader;
class TrajectoryLoader {
    constructor(store, trajectory, parentLoader) {
        this.store = store;
        this.trajectory = trajectory;
        this.parentLoader = parentLoader;
        this.stations = this.trajectory.trajectoryStation;
        trajectory.trajectoryStation = null;
    }
    loadTrajectory() {
        this.parentLoader.emit("done");
    }
}
exports.TrajectoryLoader = TrajectoryLoader;
class FileLoader extends events.EventEmitter {
    constructor(store, config) {
        super();
        this.store = store;
        this.config = config;
        this.headerOnly = false;
        this.messageData = {};
        this.log = null;
        this.trajectory = null;
        this.points = 0;
        this.rows = 0;
        this.channels = 0;
        this.stations = 0;
        this.nullValues = 0;
        config.schemaModule || (config.schemaModule = "witsml1411");
    }
    onLoadComplete() {
        this.emit("done");
    }
    start() {
        this.emit("ready");
    }
    handleLog(err, result) {
        this.dataObject.logData = this.messageData.logData;
        this.dataObject.logCurveInfo = this.messageData.logCurveInfo;
        this.log = new LogLoader(this.store, this.dataObject, this);
        this.log.headerOnly = this.headerOnly;
        this.log.loadLog();
    }
    handleTrajectory(err, result) {
        this.trajectory = new TrajectoryLoader(this.store, this.dataObject, this);
        this.trajectory.loadTrajectory();
    }
    load_doc(file) {
        var schemas = require(this.config.schemaModule)[this.config.schemaModule];
        var ctx = new Jsonix.Context([schemas]);
        var ser = ctx.createUnmarshaller();
        ser.unmarshalFile(file, function (doc) {
            var collectionName = doc.name.localPart;
            var singularName = collectionName.substr(0, collectionName.length - 1);
            var singularCollection = doc.value[singularName];
            for (var i = 0; i < singularCollection.length; i++) {
                var callback = this.onLoadComplete.bind(this);
                this.dataObject = singularCollection[i];
                switch (singularName) {
                    case "log":
                        if (this.dataObject.logData
                            && this.dataObject.logData.length > 0
                            && this.dataObject.logData[0].data != null) {
                            this.rows = this.dataObject.logData[0].data.length;
                            this.points = this.rows * this.dataObject.logCurveInfo.length;
                        }
                        if (this.dataObject.logCurveInfo)
                            this.channels = this.dataObject.logCurveInfo.length;
                        callback = this.handleLog.bind(this);
                        break;
                    case "trajectory":
                        if (this.dataObject.trajectoryStation) {
                            this.stations = this.dataObject.trajectoryStation.length;
                        }
                        callback = this.handleTrajectory.bind(this);
                        break;
                }
                if (this.store) {
                    this.messageData = {
                        header: {},
                        body: {
                            data: {
                                resource: {
                                    uri: "eml:///" + this.schemaModule + "/obj_" + singularName + "(" + this.dataObject.uid + ")",
                                    contentType: "application/x-witsml+xml;version=1.4.1.1;type=obj_" + singularName
                                }
                            }
                        },
                        nativeObject: this.dataObject
                    };
                    if (this.config.dryRun != 'true') {
                        this.store.put(this.messageData, callback);
                    }
                    else {
                        this.emit("done");
                    }
                }
                else {
                    callback(null, null);
                }
            }
        }.bind(this));
    }
}
exports.FileLoader = FileLoader;
//# sourceMappingURL=WitsmlHelper.js.map