"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var perfmon = require("perfmon");
var mkdirp = require("mkdirp");
var uuid = require("node-uuid");
const path = require("path");
const fs = require("fs");
const events = require("events");
const EnergisticsCommons_1 = require("../common/EnergisticsCommons");
const etp_1 = require("etp");
const EtpUri_1 = require("../common/EtpUri");
var appRoot = path.dirname(path.dirname(module.filename));
var WITSML_ROOT = "eml:///witsml20";
var m_sessions = [];
var m_channels = [];
var m_reverse_index = {};
var m_host;
function newData(err, data) {
    for (var i = 0; i < m_sessions.length; i++) {
        m_sessions[i].nextRow(err, data);
    }
}
function configure() {
    var configDir = appRoot + "/config/" + m_host;
    var configFile = configDir + "/config.json";
    mkdirp.sync(configDir);
    var config = {};
    if (fs.existsSync(configFile)) {
        console.log("Re-using channel configuration file: " + configFile);
        config = require(configFile);
    }
    else {
        console.log("Creating new channel configuration file: " + configFile);
    }
    for (var channel in m_reverse_index) {
        if (config.hasOwnProperty(channel) && config[channel].hasOwnProperty("uuid")) {
            m_reverse_index[channel].uuid = config[channel]["uuid"];
        }
        else {
            config[channel] = { uuid: uuid.v4() };
            m_reverse_index[channel].uuid = config[channel].uuid;
        }
    }
    fs.writeFileSync(configFile, JSON.stringify(config));
}
function startRecording() {
    configure();
    console.log("Starting recording of " + m_channels.length + " channels.");
    perfmon(m_channels, newData);
}
function registerChannel(err, data) {
    for (var i = 0; i < data.counters.length; i++) {
        var id = m_channels.length;
        m_host = data.host;
        m_channels[id] = data.counters[i];
        m_reverse_index[data.counters[i]] = { id: id, streaming: false, lastValue: null };
    }
    if (this.lastChannel) {
        startRecording();
    }
}
function validForUri(val, index) {
    var uris = this;
    for (var i = 0; i < uris.length; i++) {
        if (uris[i] == "/")
            return true;
        else {
            return (val.channelUri.indexOf(uris[i]) == 0);
        }
    }
    return true;
}
perfmon.list('memory', registerChannel);
perfmon.list('processor', registerChannel.bind({ lastChannel: true }));
class MyResource extends etp_1.Energistics.Datatypes.Object.Resource {
    constructor(uri, name, resourceType = "DataObject") {
        super();
        this.uri = uri;
        this.name = name;
        this.resourceType = resourceType;
        this.channelSubscribable = true;
        this.hasChildren = -1;
        this.customData = {};
    }
}
class PerfMonProvider extends events.EventEmitter {
    constructor(sessionId, configuration) {
        super();
        this.sessionId = sessionId;
        this.newChannelId = null;
        this.running = true;
        this.server = "";
        this.activeChannels = [];
        this.lastValue = [];
        this.config = {};
        this.config = configuration;
        m_sessions.push(this);
    }
    nextRow(err, data) {
        if (!this.running)
            return;
        var channelData = { data: [] };
        var time = new Date(data.time);
        var indexValue = EnergisticsCommons_1.etpDate(time);
        for (var counter in data.counters) {
            var channel = m_reverse_index[counter];
            var channelId = channel.id;
            var dataValue = data.counters[counter];
            if (channel && this.activeChannels[channelId]) {
                if ((this.lastValue[channelId] != dataValue) || (!this.config.skipDuplicates)) {
                    this.lastValue[channelId] = dataValue;
                    var dataItem = {
                        indexes: [],
                        channelId: channel.id,
                        value: { item: { double: dataValue } },
                        valueAttributes: []
                    };
                    channelData.data.push(dataItem);
                }
            }
        }
        if (channelData.data.length > 0) {
            channelData.data[0].indexes.push(indexValue);
            this.emit("data", channelData);
        }
    }
    startAllChannels() {
        for (var i = 0; i < m_channels.length; i++) {
            var csi = new etp_1.Energistics.Datatypes.ChannelData.ChannelStreamingInfo();
            csi.channelId = i;
            this.activeChannels[i] = csi;
        }
    }
    startStreaming(channels) {
        for (var i = 0; i < channels.length; i++) {
            var csi = channels[i];
            this.activeChannels[csi.channelId] = csi;
        }
    }
    stopStreaming(channels) {
        for (var i = 0; i < channels.length; i++) {
            delete this.activeChannels[channels[i]];
        }
    }
    stop() {
        if (this.running) {
            var index = m_sessions.indexOf(this);
            if (index >= 0) {
                console.log("Removing session {" + this.sessionId + "} from sender list.");
                m_sessions.splice(index, 1);
            }
        }
        this.running = false;
    }
    describe(msg) {
        var uris = msg.body.uris;
        var channels = m_channels.filter(validForUri, uris);
        var metadataRecords = channels.map(this.toChannelMetadata, this);
        this.emit('metadata', { channels: metadataRecords }, msg.header.messageId);
    }
    toChannelMetadata(uri) {
        if (m_channels.indexOf(uri) < 0)
            throw new Error("URI Does not exist");
        return {
            channelUri: uri,
            channelId: m_reverse_index[uri].id,
            indexes: [
                { indexType: "Time", description: "Time", uri: { string: "Time" }, direction: "Increasing", mnemonic: "Time", indexID: 0, uom: "ms", datum: null, customData: {} }
            ],
            dataType: "double",
            uom: "euc",
            startIndex: null,
            endIndex: null,
            channelName: uri,
            description: uri,
            status: "Active",
            contentType: null,
            source: m_host,
            measureClass: "time",
            uuid: m_reverse_index[uri].uuid
        };
    }
}
exports.PerfMonProvider = PerfMonProvider;
class PerfStore extends events.EventEmitter {
    constructor(connectString) {
        super();
    }
    connect() { }
    enum(uriString, callback) {
        var uri = new EtpUri_1.EtpUri(uriString);
        if (uri.isRoot) {
            callback(null, [new MyResource(WITSML_ROOT, 'WITSML Store', "UriProtocol")]);
        }
        else if (uri.isWitsml) {
            if (uriString == WITSML_ROOT) {
                callback(null, [new MyResource(WITSML_ROOT + '/obj_well', 'Wells', "Folder")]);
            }
            else if (uri.collection == "well") {
                if (uri.identifier) {
                    var retval = new MyResource(WITSML_ROOT + '/obj_wellbore(' + m_host + '-1)', 'WB 1', 'DataObject');
                    retval.contentType = "application/x-witsml+xml;version=2.0;type=obj_wellbore";
                    callback(null, [retval]);
                }
                else {
                    var retval = new MyResource(WITSML_ROOT + '/obj_well(' + m_host + ')', 'Computer - ' + m_host, 'DataObject');
                    retval.contentType = "application/x-witsml+xml;version=2.0;type=obj_well";
                    callback(null, [retval]);
                }
            }
            else if (uri.collection == "wellbore") {
                var retval1 = new MyResource(WITSML_ROOT + '/obj_log(memory)', 'Memory', 'DataObject');
                var retval2 = new MyResource(WITSML_ROOT + '/obj_log(processor)', 'Processor', 'DataObject');
                retval1.contentType = retval2.contentType = "application/x-witsml+xml;version=2.0;type=obj_log";
                callback(null, [retval1, retval2]);
            }
            else if (uri.collection == "log") {
                var results = [];
                for (var metric in m_reverse_index) {
                    if (metric.indexOf(uri.identifier) == 0) {
                        var channelId = m_reverse_index[metric].id;
                        var channel = new MyResource(WITSML_ROOT + '/obj_channel(' + channelId + ')', metric, 'DataObject');
                        channel.hasChildren = 0;
                        channel.contentType = "application/x-witsml+xml;version=2.0;type=obj_channel";
                        results.push(channel);
                    }
                }
                callback(null, results);
            }
        }
    }
    get(uriString, callback) { }
    getHistory(uri, callback) { }
    put(obj) { }
    putChannelValue(uriChannel, index, value) { }
    delete(uriString) { }
    on_connect(err, db) {
        this.emit('connect');
    }
    notificationRequest(messageData, callback) {
    }
}
exports.PerfStore = PerfStore;
//# sourceMappingURL=PerfMonProvider.js.map