"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const fs = require("fs");
const avro = require("etp-avro");
const uuid = require('node-uuid');
const etp_1 = require("etp");
const ETPCore_1 = require("../common/ETPCore");
const Core = etp_1.Energistics.Protocol.Core;
const PROTOCOL = etp_1.Energistics.Datatypes.Protocols;
const appRoot = path.dirname(path.dirname(path.dirname(module.filename)));
class ServerSession extends ETPCore_1.ETPCore {
    constructor(configuration, store = null) {
        super(configuration, store);
        this.loggedMessageId = 0;
        this.sessionId = uuid.v4();
    }
    start() {
        console.log("Starting ServerSession: ");
        if (this.config.traceMessages) {
            fs.mkdirSync(appRoot + '/trace/' + this.sessionId);
        }
        console.dir(this.config);
        console.log("Binary: ", this.binary);
        this.connection.on('message', this.onSocketMessage.bind(this));
        this.connection.on('close', this.onSocketClose.bind(this));
    }
    handleMessage(messageHeader, messageBody) {
        if (this.config.traceMessages) {
            var msgJson = JSON.stringify(["R", messageHeader, messageBody], null, 4);
            fs.writeFile(appRoot + "/trace/" + this.sessionId + "/" + (++this.loggedMessageId) + ".json", msgJson);
        }
        try {
            if (messageHeader.protocol == PROTOCOL.Core) {
                switch (messageHeader.messageType) {
                    case Core.MsgRequestSession:
                        return this.onRequestSession(messageHeader, messageBody);
                    case Core.MsgCloseSession:
                        return this.onCloseSession(messageHeader, messageBody);
                }
                return super.handleMessage(messageHeader, messageBody);
            }
            else {
                if (this.handlers[messageHeader.protocol])
                    this.handlers[messageHeader.protocol].handleMessage(messageHeader, messageBody);
                else
                    this.sendException(-1, "Unsupported protocol", messageHeader.messageId);
            }
        }
        catch (err) {
            this.log(err);
        }
    }
    onRequestSession(header, body) {
        var agreedProtocols = [];
        for (var i = 0; i < body.requestedProtocols.length; i++) {
            var protocol = body.requestedProtocols[i];
            if (this.handlers[protocol.protocol] && this.handlers[protocol.protocol].role == protocol.role) {
                agreedProtocols.push(protocol);
            }
        }
        if (this.config.simpleStreamer && agreedProtocols[0].protocol == 1) {
            agreedProtocols[0].protocolCapabilities = { simpleStreamer: { item: { boolean: true } } };
        }
        var msg = {
            applicationName: "ralf-server",
            applicationVersion: "0.0.1",
            sessionId: (this.sessionId),
            supportedProtocols: agreedProtocols
        };
        if (msg.supportedProtocols.length > 0) {
            this.log("Opening new session {" + msg.sessionId + "} with " + body.applicationName);
            this.send(this.createHeader(PROTOCOL.Core, Core.MsgOpenSession, header.messageId), msg);
        }
        else {
            this.log("No supported protocols: " + JSON.stringify(body));
            var error = { errorCode: 2, errorMessage: "No supported protocols." };
            this.send(this.createHeader(PROTOCOL.Core, Core.MsgProtocolException, header.messageId), error);
        }
        for (var i = 0; i < agreedProtocols.length; i++) {
            var protocol = agreedProtocols[i];
            this.log("Using protocol: " + protocol.protocol + " in role of '" + protocol.role + "'; + with caps: " + JSON.stringify(protocol.protocolCapabilities));
        }
    }
    onSocketClose(reasonCode, description) {
        this.log((new Date()) + ' Peer ' + this.connection.remoteAddress + ' disconnected.');
        this.log('--Reason Code [' + reasonCode + '] Description [' + description + ']');
        this.emit('disconnect');
        for (var i = 0; i < this.handlers.length; i++) {
            if (this.handlers[i])
                this.handlers[i].stop();
        }
        if (this.config.exitOnClose) {
            this.log("Exiting process normally because started with --exitOnClose.");
            process.exit(0);
        }
    }
    onSocketMessage(message, flags) {
        this.stats.messagesReceived++;
        if (message.type === 'utf8' || (flags && (!flags.binary))) {
            this.binary = false;
            this.stats.bytesReceived += message.utf8Data.length;
            var parts = JSON.parse(message.utf8Data);
            this.handleMessage(parts[0], parts[1]);
        }
        else {
            this.stats.bytesReceived += message.binaryData.length;
            var reader = new avro.BinaryReader(this.schemaCache, new Uint8Array(message.binaryData)), header = reader.readDatum("Energistics.Datatypes.MessageHeader"), body = reader.readDatum(this.schemaCache.find(header.protocol, header.messageType));
            this.handleMessage(header, body);
        }
    }
    resume() {
    }
    send(header, message) {
        var dataToSend = null, encoder = null, schemaName = this.schemaCache.find(header.protocol, header.messageType).fullName;
        if (this.config.traceMessages) {
            var msgJson = JSON.stringify(["S", header, message], null, 4);
            fs.writeFile(appRoot + "/trace/" + this.sessionId + "/" + (++this.loggedMessageId) + ".json", msgJson);
        }
        this.stats.messagesSent++;
        if (this.binary) {
            encoder = new avro.BinaryWriter(this.schemaCache);
            encoder.writeDatum("Energistics.Datatypes.MessageHeader", header);
            encoder.writeDatum(schemaName, message);
            dataToSend = encoder.getBuffer();
        }
        else {
            dataToSend = JSON.stringify([header, message]);
        }
        this.stats.bytesSent += dataToSend.length;
        this.connection.send(dataToSend, { binary: this.binary, mask: false });
    }
}
exports.ServerSession = ServerSession;
//# sourceMappingURL=ServerSession.js.map